// rcvUDPLnx.cpp (V1.0): the receiver of UDP Load in Linux.
//
// Author:  Huahui Wu
//          flashine@gmail.com
//
// History
//   Version 1.0 - 2005 Nov 11
//
// Description:
//   rcvUDPLnx.cpp
//		Very simple, Works in conjunction with sndUDPWin.cpp as 
//		a Windows sender or sndUDPLnx as a Linux sender.
//		The program sets itself up as a receiver using the UDP
//		protocol. It waits for UDP data from a sender, displays
//		the incoming connection, bounce the ping packet back 
//		to the sender.
//
// Compile and Link: 
//	 g++ rcvUDPLnx.cpp -o rcvUDPLnx
//
// Run: 
//   Pass the port number that the receiver should bind() to
//   on the command line. Any port number not already in use
//   can be specified. Example: rcvUDPLnx 1234
//
// License: 
//   This software is released into the public domain.
//   You are free to use it in any way you like.
//   This software is provided "as is" with no expressed
//   or implied warranty.  I accept no liability for any
//   damage or loss of business that this software may cause.
//
///////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#define MAX_PKT_SIZE 10240
#define MAX_PKT_NUM 1000

struct Ping_Pkt{
        int seq;
        int nPkt;
        unsigned char padding[MAX_PKT_SIZE];
};

//global variables
int theSocket;

// Function prototype
void DatagramReceiver(short nPort);	// the receiver 
void CleanUp(int);		     // press ctrl_c to cleanup	
int GetCurTimeInMs();

int pktRcvd;
int initCounter; // the counter when PING starts
int prevCounter; // the counter of previous packet
int  curCounter;
int  frequency;
int minDelay, maxDelay; // the minum and the maximum delay


int main(int argc, char **argv)
{
	int nRet;
	short nPort;

	//
	// Check for port argument
	//
	if (argc == 2)
	  nPort = atoi(argv[1]);
	else
	  nPort = 1234;
	
	signal(SIGINT, CleanUp);
	//
	// Do all the stuff a datagram receiver does
	//
	DatagramReceiver(nPort);
	
	return (0);	
}

////////////////////////////////////////////////////////////

void DatagramReceiver(short nPort)
{
    //
    // Create a UDP/IP datagram socket
    //
    if ((theSocket = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	perror("can't open stream socket");
	exit(1);
    }
    
    // Fill in the address structure
    //
    struct sockaddr_in saReceiver;
    bzero((char *) &saReceiver, sizeof(saReceiver));
    saReceiver.sin_family = AF_INET;
    saReceiver.sin_addr.s_addr 
	= htonl(INADDR_ANY); // Let socket assign address
    saReceiver.sin_port = htons(nPort); // Use port passed from user
    
    
    //
    // bind the name to the socket
    //
    int nRet;
    nRet = bind(theSocket,		// Socket descriptor
		(struct sockaddr *)&saReceiver,  // Address to bind to
		sizeof(saReceiver)	// Size of address
	);
    if (nRet < 0)	{
	perror("Can't bind to local address");
	exit(1);
    }
    
    
    //
    // This isn't normally done or required, but in this 
    // example we're printing out where the receiver is waiting
    // so that you can connect the example sender.
    //
    int nLen;
    nLen = sizeof(sockaddr);
    char szBuf[4096];
    
    nRet = gethostname(szBuf, sizeof(szBuf));
    if (nRet < 0)
    {
	perror("gethostname");
	exit(1);
    }
    
    //
    // Show the receiver name and port number
    //
    printf("\nReceiver named %s waiting on port %d\n",
	   szBuf, nPort);
    
    
    //
    // Wait for data from the sender
    //
    struct sockaddr_in saSender;
    double tInterval;
    
    while(1){
	memset(szBuf, 0, sizeof(szBuf));
	nRet = recvfrom(theSocket,  // Bound socket
			szBuf,      // Receive buffer
			sizeof(szBuf),// Size of buffer in bytes
			0, // Flags
			(struct sockaddr *)&saSender,// Buffer
			(socklen_t *)&nLen); // Length of sender address
	
	Ping_Pkt* ping_pkt=(Ping_Pkt*)szBuf;
	if (ping_pkt->seq < 0) //the sender stopped
	    CleanUp(1);
	//
	// Display the data that was received once per second
	//
	if (ping_pkt->seq % ping_pkt->nPkt == 0){
	    curCounter = GetCurTimeInMs();
	    
	    if (initCounter == 0){ //if this is the first packet
		  initCounter = curCounter;
	    }
	    
	    //get the time between two display
	      tInterval = curCounter - prevCounter;
	      if (tInterval < 1)
		  tInterval = 1;
	      
	      printf("From sender: size = %d bytes pkt recved=%d  bw=%8.3f KB\n",
		     nRet, pktRcvd, 1.0*ping_pkt->nPkt*nRet/tInterval
		  );
	      
	      prevCounter = curCounter;
	      
	}
	pktRcvd ++;
	
    }
    close(theSocket);
    return;
    
}

void CleanUp(int arg1){
        close(theSocket);
	printf("Receiver is now off\n");
	exit(0);
}

int GetCurTimeInMs(){
  struct timeval tv;
  gettimeofday(&tv,NULL);
  return (tv.tv_sec*1000+tv.tv_usec/1000);
}
