// sndUDPLnx.cpp (V1.0): the sender of UDPLoad in Linux
//
// Author:  Huahui Wu
//          flashine@gmail.com
//
// History
//   Version 1.0 - 2005 Nov 10
//
// Description:
//	 sndUDPLnx.cpp
//		Very simple, Works in conjunction with rcvUDPWin.cpp 
//		(a Windows receiver) or rcvUDPLnx (a Linux receiver).
//		The program attempts to connect to the receiver and port
//		specified on the command line. The sender keeps sending 
//		packets to the receiver with a high load.
//
// Compile and Link: 
//	 g++ sndUDPLnx.cpp -o sndUDPLnx
//
// Run (Usage): 
//		sndUDPLnx -p portnumber -h hostname 
//			-s packet_size_in_bytes -n packet_number_per_second
//		where the default values are:
//			hostname	: localhost		
//			portnumber	:	1234
//			packet_size	:	1024
//			packet_number:	100
//
// License: 
//   This software is released into the public domain.
//   You are free to use it in any way you like.
//   This software is provided "as is" with no expressed
//   or implied warranty.  I accept no liability for any
//   damage or loss of business that this software may cause.
//
//


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <netdb.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#ifndef INADDR_NONE
#define INADDR_NONE 0xffffffff /* should be in <netinet/in.h> */
#endif


#define MAX_PKT_SIZE 10240
#define MAX_PKT_NUM 10000

struct Ping_Pkt{
  int seq;
  int nPkt;
  unsigned char padding[MAX_PKT_SIZE];
};

//global variables
int pkt_sent=0, pkt_rcvd=0;
double p; //loss rate
int rtt_min = 2000, rtt_max = 0;
double rtt_avg = 0;
char* nmReceiver; //receiver name
short nPort; //port number
short sPkt;  //packet size in bytes
short nPkt;  //packet number per second
int initMs; // the time in ms when program starts 

// Function prototype
void DatagramSender(char *szReceiver, short nPort);
void *SendFunc(void *); 
int GetCurTimeInMs();
void CleanUp(int);
int magic_number;
int GetMagicNumber(); // get the magic number whose loop 
                      // takes 1 micro second

int	theSocket;
struct sockaddr_in saReceiver;


////////////////////////////////////////////////////////////

int main(int argc, char **argv)
{
  int nRet;
  
  //use getopt to parse the command line
  // > sndUDPLnx -p portnumber -h hostname -s packet_size_in_bytes -n packet_number_per_second
  // where the default values are:
  //		portnumber	:	1234
  //		packet_size	:	1024
  //		packet_number:	100
  int c;
  nmReceiver="localhost";
  nPort = 1234; //port number
  sPkt = 1024;  //minimum packet size in bytes
  nPkt = 100;  //packet number per second
  
  while ((c = getopt(argc, argv, "p:h:s:n:")) != EOF)
    {
      switch (c)
        {
	case 'p':
	  nPort = atoi(optarg);
	  printf("portnumber: %d\n", atoi(optarg));
	  break;
	  
	case 'h':
	  nmReceiver = optarg;
	  printf("hostname: %s\n", optarg);
	  break;
	  
	case 's':
	  sPkt = atoi(optarg);
	  if (sPkt < sizeof(int)*2)
	    sPkt = sizeof(int)*2;
	  if (sPkt > MAX_PKT_SIZE)
	    sPkt = MAX_PKT_SIZE;	
	  printf("packet size: %d\n", atoi(optarg));
	  break;
	  
	case 'n':
	  nPkt = atoi(optarg);
	  if (nPkt > MAX_PKT_NUM)
	    nPkt = MAX_PKT_NUM;
	  printf("packet number per second: %d\n", atoi(optarg));
	  break;
	  
	case '?':
	  printf("ERROR: illegal option %s\n", argv[optind-1]);
	  printf("Usage:\n");
	  printf("\t%s -p portnumber -h hostname -s packet_size_in_bytes -n packet_number_per_second\n", argv[0]);
	  exit(1); 
	  break;
	  
	default:
	  printf("WARNING: no handler for option %c\n", c);
	  printf("Usage:\n");
	  printf("\t%s -p portnumber -h hostname -s packet_size_in_bytes -n packet_number_per_second\n", argv[0]);
	  exit(1);
	  break;
        }
    }


  // Go do all the stuff a datagram sender does
  //
  signal(SIGINT, CleanUp);

  magic_number = GetMagicNumber();
  printf("Magic Number: %d\n", magic_number);
  
  DatagramSender(nmReceiver, nPort);
  
}

////////////////////////////////////////////////////////////

void DatagramSender(char *szReceiver, short nPort)
{
  struct hostent *hp;

  printf("Send %s with %d bytes of data:\n\n", szReceiver, sPkt); 
  
  //
  // Find the receiver
  //
  // Convert the host name as a dotted-decimal number.

   bzero((void *) &saReceiver, sizeof(saReceiver));
   printf("Looking up %s...\n", szReceiver);
   if ((hp = gethostbyname(szReceiver)) == NULL) {
     perror("host name error");
     exit(1);
   }
   bcopy(hp->h_addr, (char *) &saReceiver.sin_addr, hp->h_length);

   //
   // Create a UDP/IP datagram socket
   //
   theSocket = socket(AF_INET,			// Address family
		      SOCK_DGRAM,		// Socket type
		      IPPROTO_UDP);	// Protocol
   if (theSocket < 0){
       perror("Failed in creating socket");
       exit(1);
   }
   
   //
   // Fill in the address structure for the receiver
   //
   saReceiver.sin_family = AF_INET;
   saReceiver.sin_port = htons(nPort);	// Port number from command line

   Ping_Pkt ping_pkt; 
   int nRet;

   ping_pkt.seq = 0; //prepare the first packet
   initMs = GetCurTimeInMs();
  
  for (int i =0; ; i ++){
    ping_pkt.seq = i;
    ping_pkt.nPkt = nPkt;
    
    nRet = sendto(theSocket,		// Socket
		  (const char*)&ping_pkt,	// Data buffer
		  sPkt,			// Length of data
		  0,			// Flags
		  (struct sockaddr *)&saReceiver,	// Receiver address
		  sizeof(struct sockaddr)); // Length of address
    if (nRet < 0)
      {
	perror("sending");
	close(theSocket);
	exit(1);
      }

    pkt_sent ++;

    if (i%nPkt == 0)
      printf("Sending packet #%d... \n", i);

    double tInterval = 1000.0/nPkt; //in ms
    
    if (tInterval >= 5) //wait for sending next packet
      {
        int startMs;
	int loopMs;
	
	startMs = GetCurTimeInMs();
	
	double tWaited = 0;
	do{ 
	  loopMs = GetCurTimeInMs();
	  tWaited = loopMs - startMs;
	} while (tWaited < tInterval);
      }
    else
      {
	for (int jj=0; jj < tInterval*500*magic_number; jj++);
      }
  } //end of for()
  close(theSocket);
} 

void CleanUp(int arg1){
  int tTotal; // the time when each packet was sent in millisecond
  int curMs;

  Ping_Pkt ping_pkt; 
  int nRet;
  ping_pkt.seq = -1;
  nRet = sendto(theSocket,		// Socket
		(const char*)&ping_pkt,	// Data buffer
		sPkt,			// Length of data
		0,			// Flags
		(struct sockaddr *)&saReceiver,	// Receiver address
		sizeof(struct sockaddr)); // Length of address
  if (nRet < 0)
    {
      perror("sending");
      close(theSocket);
      exit(1);
    }
  
  //get cpu frequency and current counter
  curMs = GetCurTimeInMs();
  
  //get tTotal in millisecond
  tTotal =curMs - initMs;
  
  //report
  printf("%d packets are sent in %d ms\n", pkt_sent, tTotal);
  printf("\tAverage sending bandwidth is %8.3f packets per second\n", 1000.0*pkt_sent/tTotal);
  
  close(theSocket);
  exit(0);
}

int GetCurTimeInMs(){
  struct timeval tv;
  gettimeofday(&tv,NULL);
  return (tv.tv_sec*1000+tv.tv_usec/1000);
}

//get the magic number, whose loop takes about 1 ms
int GetMagicNumber(){
  int i;
  int startUs, endUs; // in micro second
  struct timeval tv;

  gettimeofday(&tv,NULL);
  startUs=tv.tv_sec*1000000+tv.tv_usec;

  for (i=0;i<1000000;i++); //loop for 1 million iterations

  gettimeofday(&tv,NULL);
  endUs=tv.tv_sec*1000000+tv.tv_usec;

  return (1000000/(endUs-startUs)); 
}


  
