// sndUDPWin.cpp (V1.0): the sender of UDPLoad in MS windows.
//
// Author:  Huahui Wu
//          flashine@gmail.com
//
// History
//   Version 1.0 - 2005 Oct 11
//
// Description:
//	 sndUDPWin.cpp
//		Very simple, Works in conjunction with rcvUDPWin.cpp 
//		(a Windows receiver) or rcvUDPLnx (a Linux receiver).
//		The program attempts to connect to the receiver and port
//		specified on the command line. The client keeps sending 
//		packets to the receiver and receiving the bounced packets 
//		from receiver and calculating the time difference.
//	 XGetopt.h XGetopt.cpp
//		Hans Dietrich's implementation of getopt(), a function 
//		to parse command lines. Check the code for more details 		
//
// Compile and Link: 
//	 Use MS Visual C++ .NET
//	 Compile and link with wsock32.lib
//
// Run (Usage): 
//		sndUDPWin -p portnumber -h hostname 
//			-s packet_size_in_bytes -n packet_number_per_second
//		where the default values are:
//			hostname	: localhost		
//			portnumber	:	1234
//			packet_size	:	1024
//			packet_number:	100
//
// License: 
//   This software is released into the public domain.
//   You are free to use it in any way you like.
//   This software is provided "as is" with no expressed
//   or implied warranty.  I accept no liability for any
//   damage or loss of business that this software may cause.
//
//

#include "stdafx.h"
#include <stdio.h>
#include <string.h>
#include <winsock.h>
#include <signal.h>
#include "XGetopt.h"

#define MAX_PKT_SIZE 10240
#define MAX_PKT_NUM 10000

struct Ping_Pkt{
	int seq;
	int nPkt;
	unsigned char padding[MAX_PKT_SIZE];
};

//global variables
int pkt_sent=0, pkt_rcvd=0;
double p; //loss rate
int rtt_min = 2000, rtt_max = 0;
double rtt_avg = 0;
char* nmServer; //receiver name
short nPort; //port number
short sPkt;  //packet size in bytes
short nPkt;  //packet number per second
ULARGE_INTEGER initCounter; // the counter when PING starts
bool oneway=true; // one way delay or round-trip ping?

// Function prototype
void DatagramClient(char *szServer, short nPort);
DWORD WINAPI SendFunc( LPVOID lpParam ); 
void CleanUp(int);
ULARGE_INTEGER GetCurTimeInMs(); //get current time in milli second
int magic_number;
int GetMagicNumber(); // get the magic number whose loop 
                      // takes 1 micro second

LPHOSTENT lpHostEntry;
SOCKET	theSocket;
SOCKADDR_IN saServer;


// Helper macro for displaying errors
#define PRINTERROR(s)	\
		fprintf(stderr,"\n%s: %d\n", s, WSAGetLastError())

////////////////////////////////////////////////////////////

void main(int argc, char **argv)
{
	WORD wVersionRequested = MAKEWORD(1,1);
	WSADATA wsaData;
	int nRet;

	//use getopt to parse the command line
	// > sndUDPWin.exe -p portnumber -h hostname -s packet_size_in_bytes -n packet_number_per_second
	// where the default values are:
	//		portnumber	:	1234
	//		packet_size	:	1024
	//		packet_number:	100
	int c;
	nmServer="localhost";
	nPort = 1234; //port number
	sPkt = 1024;  //packet size in bytes
	nPkt = 100;  //packet number per second

	while ((c = getopt(argc, argv, _T("p:h:s:n:"))) != EOF)
    {
		switch (c)
        {
			case _T('p'):
				nPort = atoi(optarg);
				printf("portnumber: %d\n", atoi(optarg));
                break;

            case _T('h'):
				nmServer = optarg;
				printf("hostname: %s\n", optarg);
                break;

            case _T('s'):
                sPkt = atoi(optarg);
				if (sPkt > MAX_PKT_SIZE)
					sPkt = MAX_PKT_SIZE;
				printf("packet size: %d\n", atoi(optarg));
				break;

			case _T('n'):
                nPkt = atoi(optarg);
				if (nPkt > MAX_PKT_NUM)
					nPkt = MAX_PKT_NUM;
                printf("packet number per second: %d\n", atoi(optarg));
				break;

            case _T('?'):
	            printf("ERROR: illegal option %s\n", argv[optind-1]);
				printf("Usage:\n");
				printf("\t%s -p portnumber -h hostname -s packet_size_in_bytes -n packet_number_per_second\n", argv[0]);
				exit(1); 
				break;

            default:
                printf("WARNING: no handler for option %c\n", c);
				printf("Usage:\n");
				printf("\t%s -p portnumber -h hostname -s packet_size_in_bytes -n packet_number_per_second\n", argv[0]);
                exit(1);
                break;
        }
    }

	//
	// Initialize WinSock and check the version
	//
	nRet = WSAStartup(wVersionRequested, &wsaData);
	if (wsaData.wVersion != wVersionRequested)
	{	
		fprintf(stderr,"\n Wrong version\n");
		return;
	}

	//
	// Go do all the stuff a datagram client does
	//
	signal(SIGINT, CleanUp);

	magic_number = GetMagicNumber();
	printf("Magic Number: %d\n", magic_number);

	DatagramClient(nmServer, nPort);
	
	//
	// Release WinSock
	//
	WSACleanup();
}

////////////////////////////////////////////////////////////

void DatagramClient(char *szServer, short nPort)
{

	printf("Pinging %s with 32 bytes of data:\n\n", szServer);	

	//
	// Find the receiver
	//

	lpHostEntry = gethostbyname(szServer);
    if (lpHostEntry == NULL)
    {
        PRINTERROR("gethostbyname()");
        return;
    }


	//
	// Create a UDP/IP datagram socket
	//
	theSocket = socket(AF_INET,			// Address family
					   SOCK_DGRAM,		// Socket type
					   IPPROTO_UDP);	// Protocol
	if (theSocket == INVALID_SOCKET)
	{
		PRINTERROR("socket()");
		return;
	}

	//
	// Fill in the address structure for the receiver
	//
	saServer.sin_family = AF_INET;
	saServer.sin_addr = *((LPIN_ADDR)*lpHostEntry->h_addr_list);
										// ^ receiver's address
	saServer.sin_port = htons(nPort);	// Port number from command line

   	Ping_Pkt ping_pkt; 
	int nRet;

	pkt_sent = 0;
	initCounter = GetCurTimeInMs();

	for (int i =0; ; i ++){
		ping_pkt.seq = i;
		ping_pkt.nPkt = nPkt;

		pkt_sent ++;
		nRet = sendto(theSocket,				// Socket
					  (const char*)&ping_pkt,	// Data buffer
					  sPkt,			// Length of data
					  0,						// Flags
					  (LPSOCKADDR)&saServer,	// receiver address
					sizeof(struct sockaddr)); // Length of address
		if (nRet == SOCKET_ERROR)
		{
			PRINTERROR("sendto()");
			closesocket(theSocket);
			return;
		}

		//the time in ms we need wait before sending next packet
		double tInterval = 1000.0 / nPkt;

		//debug code
		if (i%nPkt == 0)
			printf("Sending packet #%d... \n", i);

		if (tInterval >= 5) //wait for sending next packet
		{
			LARGE_INTEGER   startCounter;
			LARGE_INTEGER   loopCounter;
			LARGE_INTEGER	frequency;
			::QueryPerformanceCounter(&startCounter);
			::QueryPerformanceFrequency(&frequency);

			double tWaited = 0;
			do{ 
				::QueryPerformanceCounter(&loopCounter);
				tWaited = (loopCounter.QuadPart - startCounter.QuadPart)*1000.0/
					frequency.QuadPart;
			} while (tWaited <= tInterval);
		}else{
			for (int jj=0; jj < tInterval*500*magic_number; jj++);
		}
		
	}
	closesocket(theSocket);
	return;
}


void CleanUp(int arg1){
	int tTotal; // the time when each packet was sent in millisecond
	ULARGE_INTEGER curCounter;

	Ping_Pkt ping_pkt; 
	int nRet;
	ping_pkt.seq = -1;
	nRet = sendto(theSocket,		// Socket
		(const char*)&ping_pkt,	// Data buffer
		sPkt,			// Length of data
		0,			// Flags
		(struct sockaddr *)&saServer,	// Receiver address
		sizeof(struct sockaddr)); // Length of address
	if (nRet < 0)
	{
      perror("sending");
      closesocket(theSocket);
      exit(1);
    }

	//get cpu frequency and current counter
	curCounter = GetCurTimeInMs();

	//get tTotal in millisecond
	tTotal =curCounter.QuadPart - initCounter.QuadPart;

	//report
	printf("sndUDP Statistics for %s:\n", nmServer);
	printf("\t%d packets are sent in %d ms\n", pkt_sent, tTotal);
	printf("\tAverage sending bandwidth is %8.3f packets per second\n", 1000.0*pkt_sent/tTotal);

	exit(0);
}

ULARGE_INTEGER GetCurTimeInMs(){
	SYSTEMTIME  sysTime;
	FILETIME	fileTime;
	ULARGE_INTEGER timeInMs;
	::GetSystemTime(&sysTime);
	::SystemTimeToFileTime(&sysTime,&fileTime);
	timeInMs.LowPart = fileTime.dwLowDateTime;
	timeInMs.HighPart = fileTime.dwHighDateTime;
	timeInMs.QuadPart /= 10000; // the filetime is 100-nanosecond based.
								// but we need milli second based
	return timeInMs;
}

//get the magic number, whose loop takes about 1 micro sec
int GetMagicNumber(){
  int i;
	LARGE_INTEGER   startCounter;
	LARGE_INTEGER   endCounter;
	LARGE_INTEGER	frequency;
	double tLooped;

	::QueryPerformanceCounter(&startCounter);
	::QueryPerformanceFrequency(&frequency);

	for (i=0;i<1000000;i++); //loop for 1 million iterations

	::QueryPerformanceCounter(&endCounter);
	tLooped = 1000000*(endCounter.QuadPart - startCounter.QuadPart)/frequency.QuadPart;

	return (1000/(tLooped)); 
}

